@@ -1,4 +1,4 @@
-/* cups-pdf.c -- CUPS Backend (version 2.5.0, 2009-01-26)
+/* cups-pdf.c -- CUPS Backend (version 2.5.1, 2011-02-18)
    08.02.2003, Volker C. Behr
    Experimentelle Physik V, Universitaet Wuerzburg 
    behr@physik.uni-wuerzburg.de
@@ -12,7 +12,7 @@
 
    ---------------------------------------------------------------------------
 
-   Copyright (C) 2003-2009  Volker C. Behr
+   Copyright (C) 2003-2011  Volker C. Behr
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -71,11 +71,11 @@ static void log_event(short type, char message[], char *detail) {
     timestring[strlen(timestring)-1]='\0';
 
     if (type == CPERROR) 
-      sprintf(ctype, "ERROR");
+      snprintf(ctype, 8, "ERROR");
     else if (type == CPSTATUS)
-      sprintf(ctype, "STATUS");
+      snprintf(ctype, 8, "STATUS");
     else
-      sprintf(ctype, "DEBUG");
+      snprintf(ctype, 8, "DEBUG");
     if (detail != NULL)  {
       while (detail[strlen(detail)-1] == '\n') 
         detail[strlen(detail)-1]='\0';
@@ -85,7 +85,10 @@ static void log_event(short type, char message[], char *detail) {
     }
     else
       fprintf(logfp,"%s  [%s] %s\n", timestring, ctype, message);
+    
+    (void) fflush(logfp);
   }
+
   return;
 }
 
@@ -478,37 +481,48 @@ static int preparespoolfile(FILE *fpsrc, char *spoolfile, char *title, char *cmd
   (void) fclose(fpsrc);
   log_event(CPDEBUG, "all data written to spoolfile", spoolfile);
 
-  if (!strcmp(cmdtitle, "(stdin)")) 
-    cmdtitle="";
+  if (cmdtitle == NULL || !strcmp(cmdtitle, "(stdin)")) 
+    buffer[0]='\0';
+  else
+    strncpy(buffer, cmdtitle, BUFSIZE);
+  if (title == NULL || !strcmp(title, "((stdin))")) 
+    title[0]='\0';
+
   if (conf.titlepref) {
-    log_event(CPDEBUG, "trying to use commandline title", cmdtitle);
-    if (!preparetitle(cmdtitle)) {
+    log_event(CPDEBUG, "trying to use commandline title", buffer);
+    if (!preparetitle(buffer)) {
       log_event(CPDEBUG, "empty commandline title, using PS title", title);
       if (!preparetitle(title))
         log_event(CPDEBUG, "empty PS title", NULL); 
     }
     else
-      snprintf(title, BUFSIZE, "%s", cmdtitle);
+      snprintf(title, BUFSIZE, "%s", buffer);
   }
   else {
     log_event(CPDEBUG, "trying to use PS title", title);
     if (!preparetitle(title)) {
-      log_event(CPDEBUG, "empty PS title, using commandline title", cmdtitle);
-      if (!preparetitle(cmdtitle))
+      log_event(CPDEBUG, "empty PS title, using commandline title", buffer);
+      if (!preparetitle(buffer))
         log_event(CPDEBUG, "empty commandline title", NULL);
       else
-        snprintf(title, BUFSIZE, "%s", cmdtitle);
+        snprintf(title, BUFSIZE, "%s", buffer);
     }
   }          
 
   if (!strcmp(title, "")) { 
-    snprintf(title, BUFSIZE, "job_%i-untitled_document", job);
+    if (conf.label == 2)
+      snprintf(title, BUFSIZE, "untitled_document-job_%i", job);
+    else
+      snprintf(title, BUFSIZE, "job_%i-untitled_document", job);
     log_event(CPDEBUG, "no title found - using default value", title);
   }
   else {
     if (conf.label) {
       strcpy(buffer, title);
-      snprintf(title, BUFSIZE, "job_%i-%s", job, buffer);
+      if (conf.label == 2)
+        snprintf(title, BUFSIZE, "%s-job_%i", buffer, job);
+      else
+        snprintf(title, BUFSIZE, "job_%i-%s", job, buffer);
     }
     log_event(CPDEBUG, "title successfully retrieved", title);
   }
@@ -517,7 +531,7 @@ static int preparespoolfile(FILE *fpsrc, char *spoolfile, char *title, char *cmd
 
 int main(int argc, char *argv[]) {
   char *user, *dirname, *spoolfile, *outfile, *gscall, *ppcall;
-  cp_string title="";
+  cp_string title;
   int size;
   mode_t mode;
   struct passwd *passwd;
@@ -700,8 +714,6 @@ int main(int argc, char *argv[]) {
   }
   log_event(CPDEBUG, "TMPDIR set for GhostScript", getenv("TMPDIR"));
 
-  if (logfp!=NULL)
-    (void) fflush(logfp);
   pid=fork();
 
   if (!pid) {
@@ -718,7 +730,7 @@ int main(int argc, char *argv[]) {
      
     (void) umask(0077);
     size=system(gscall);
-    snprintf(title,BUFSIZE,"%d",size);
+    (void) snprintf(title,BUFSIZE,"%d",size);
     log_event(CPDEBUG, "ghostscript has finished", title);
     if (chmod(outfile, mode))
       log_event(CPERROR, "failed to set file mode for PDF file (non fatal)", outfile);
